use futures::{Stream, StreamExt};
use redis::RedisError;

#[derive(Clone)]
pub struct FirehoseFactory {
    redis_url: String,
}

impl FirehoseFactory {
    pub fn new(redis_url: String) -> Self {
        FirehoseFactory { redis_url }
    }

    pub async fn produce(&self) -> Result<impl Stream<Item = Result<Vec<u8>, RedisError>>, RedisError> {
        let client = redis::Client::open(&self.redis_url[..])?;
        let mut pubsub_conn = client.get_async_connection()
            .await?
            .into_pubsub();

        pubsub_conn.subscribe("firehose")
            .await?;

        let stream = pubsub_conn.into_on_message()
            .map(|msg| msg.get_payload());
        Ok(stream)
    }
}
